@@ -40,7 +40,7 @@
 #define LX200_GENERIC_SLEWRATE 5        /* slew rate, degrees/s */
 #define SIDRATE  0.004178 /* sidereal rate, degrees/s */
 
-LX200Telescope::LX200Telescope() : FI(this)
+LX200Telescope::LX200Telescope() : GI(this), FI(this)
 {
 }
 
@@ -145,7 +145,7 @@ bool LX200Telescope::initProperties()
 
     TrackState = SCOPE_IDLE;
 
-    initGuiderProperties(getDeviceName(), GUIDE_TAB);
+    GI::initProperties(GUIDE_TAB);
 
     /* Add debug/simulation/config controls so we may debug driver if necessary */
     addAuxControls();
@@ -218,8 +218,8 @@ bool LX200Telescope::updateProperties()
             defineProperty(&SiteNameTP);
         }
 
-        defineProperty(&GuideNSNP);
-        defineProperty(&GuideWENP);
+        GI::updateProperties();
+
 
         if (genericCapability & LX200_HAS_FOCUS)
         {
@@ -246,8 +246,7 @@ bool LX200Telescope::updateProperties()
             deleteProperty(SiteNameTP.name);
         }
 
-        deleteProperty(GuideNSNP.name);
-        deleteProperty(GuideWENP.name);
+        GI::updateProperties();
 
         if (genericCapability & LX200_HAS_FOCUS)
         {
@@ -297,9 +296,9 @@ bool LX200Telescope::ReadScopeStatus()
         if (isSlewComplete())
         {
             // Set slew mode to "Centering"
-            IUResetSwitch(&SlewRateSP);
-            SlewRateS[SLEW_CENTERING].s = ISS_ON;
-            IDSetSwitch(&SlewRateSP, nullptr);
+            SlewRateSP.reset();
+            SlewRateSP[SLEW_CENTERING].setState(ISS_ON);
+            SlewRateSP.apply();
 
             TrackState = SCOPE_TRACKING;
             LOG_INFO("Slew is complete. Tracking...");
@@ -315,8 +314,9 @@ bool LX200Telescope::ReadScopeStatus()
 
     if (getLX200RA(PortFD, &currentRA) < 0 || getLX200DEC(PortFD, &currentDEC) < 0)
     {
-        EqNP.s = IPS_ALERT;
-        IDSetNumber(&EqNP, "Error reading RA/DEC.");
+        EqNP.setState(IPS_ALERT);
+        LOG_ERROR("Error reading RA/DEC.");
+        EqNP.apply();
         return false;
     }
 
@@ -350,29 +350,31 @@ bool LX200Telescope::Goto(double ra, double dec)
     fs_sexa(DecStr, targetDEC, 2, fracbase);
 
     // If moving, let's stop it first.
-    if (EqNP.s == IPS_BUSY)
+    if (EqNP.getState() == IPS_BUSY)
     {
         if (!isSimulation() && abortSlew(PortFD) < 0)
         {
-            AbortSP.s = IPS_ALERT;
-            IDSetSwitch(&AbortSP, "Abort slew failed.");
+            AbortSP.setState(IPS_ALERT);
+            LOG_ERROR("Abort slew failed.");
+            AbortSP.apply();
             return false;
         }
 
-        AbortSP.s = IPS_OK;
-        EqNP.s    = IPS_IDLE;
-        IDSetSwitch(&AbortSP, "Slew aborted.");
-        IDSetNumber(&EqNP, nullptr);
+        AbortSP.setState(IPS_OK);
+        EqNP.setState(IPS_IDLE);
+        LOG_ERROR("Slew aborted.");
+        AbortSP.apply();
+        EqNP.apply();
 
-        if (MovementNSSP.s == IPS_BUSY || MovementWESP.s == IPS_BUSY)
+        if (MovementNSSP.getState() == IPS_BUSY || MovementWESP.getState() == IPS_BUSY)
         {
-            MovementNSSP.s = IPS_IDLE;
-            MovementWESP.s = IPS_IDLE;
-            EqNP.s = IPS_IDLE;
-            IUResetSwitch(&MovementNSSP);
-            IUResetSwitch(&MovementWESP);
-            IDSetSwitch(&MovementNSSP, nullptr);
-            IDSetSwitch(&MovementWESP, nullptr);
+            MovementNSSP.setState(IPS_IDLE);
+            MovementWESP.setState(IPS_IDLE);
+            EqNP.setState(IPS_IDLE);
+            MovementNSSP.reset();
+            MovementWESP.reset();
+            MovementNSSP.apply();
+            MovementWESP.apply();
         }
 
         // sleep for 100 mseconds
@@ -383,8 +385,9 @@ bool LX200Telescope::Goto(double ra, double dec)
     {
         if (setObjectRA(PortFD, targetRA) < 0 || (setObjectDEC(PortFD, targetDEC)) < 0)
         {
-            EqNP.s = IPS_ALERT;
-            IDSetNumber(&EqNP, "Error setting RA/DEC.");
+            EqNP.setState(IPS_ALERT);
+            LOG_ERROR("Error setting RA/DEC.");
+            EqNP.apply();
             return false;
         }
 
@@ -413,15 +416,17 @@ bool LX200Telescope::Sync(double ra, double dec)
 
     if (!isSimulation() && (setObjectRA(PortFD, ra) < 0 || (setObjectDEC(PortFD, dec)) < 0))
     {
-        EqNP.s = IPS_ALERT;
-        IDSetNumber(&EqNP, "Error setting RA/DEC. Unable to Sync.");
+        EqNP.setState(IPS_ALERT);
+        LOG_ERROR("Error setting RA/DEC. Unable to Sync.");
+        EqNP.apply();
         return false;
     }
 
     if (!isSimulation() && ::Sync(PortFD, syncString) < 0)
     {
-        EqNP.s = IPS_ALERT;
-        IDSetNumber(&EqNP, "Synchronization failed.");
+        EqNP.setState(IPS_ALERT);
+        LOG_ERROR("Synchronization failed.");
+        EqNP.apply();
         return false;
     }
 
@@ -430,7 +435,7 @@ bool LX200Telescope::Sync(double ra, double dec)
 
     LOG_INFO("Synchronization successful.");
 
-    EqNP.s     = IPS_OK;
+    EqNP.setState(IPS_OK);
 
     NewRaDec(currentRA, currentDEC);
 
@@ -443,30 +448,32 @@ bool LX200Telescope::Park()
     if (!isSimulation())
     {
         // If scope is moving, let's stop it first.
-        if (EqNP.s == IPS_BUSY)
+        if (EqNP.getState() == IPS_BUSY)
         {
             if (!isSimulation() && abortSlew(PortFD) < 0)
             {
-                AbortSP.s = IPS_ALERT;
-                IDSetSwitch(&AbortSP, "Abort slew failed.");
+                AbortSP.setState(IPS_ALERT);
+                LOG_ERROR("Abort slew failed");
+                AbortSP.apply();
                 return false;
             }
 
-            AbortSP.s = IPS_OK;
-            EqNP.s    = IPS_IDLE;
-            IDSetSwitch(&AbortSP, "Slew aborted.");
-            IDSetNumber(&EqNP, nullptr);
+            AbortSP.setState(IPS_OK);
+            EqNP.setState(IPS_IDLE);
+            LOG_ERROR("Slew aborted.");
+            AbortSP.apply();
+            EqNP.apply();
 
-            if (MovementNSSP.s == IPS_BUSY || MovementWESP.s == IPS_BUSY)
+            if (MovementNSSP.getState() == IPS_BUSY || MovementWESP.getState() == IPS_BUSY)
             {
-                MovementNSSP.s = IPS_IDLE;
-                MovementWESP.s = IPS_IDLE;
-                EqNP.s = IPS_IDLE;
-                IUResetSwitch(&MovementNSSP);
-                IUResetSwitch(&MovementWESP);
+                MovementNSSP.setState(IPS_IDLE);
+                MovementWESP.setState(IPS_IDLE);
+                EqNP.setState(IPS_IDLE);
+                MovementNSSP.reset();
+                MovementWESP.reset();
 
-                IDSetSwitch(&MovementNSSP, nullptr);
-                IDSetSwitch(&MovementWESP, nullptr);
+                MovementNSSP.apply();
+                MovementWESP.apply();
             }
 
             // sleep for 100 msec
@@ -475,8 +482,9 @@ bool LX200Telescope::Park()
 
         if (!isSimulation() && slewToPark(PortFD) < 0)
         {
-            ParkSP.s = IPS_ALERT;
-            IDSetSwitch(&ParkSP, "Parking Failed.");
+            ParkSP.setState(IPS_ALERT);
+            LOG_ERROR("Parking Failed.");
+            ParkSP.apply();
             return false;
         }
     }
@@ -557,11 +565,14 @@ bool LX200Telescope::Abort()
         return false;
     }
 
-    if (GuideNSNP.s == IPS_BUSY || GuideWENP.s == IPS_BUSY)
+    if (GuideNSNP.getState() == IPS_BUSY || GuideWENP.getState() == IPS_BUSY)
     {
-        GuideNSNP.s = GuideWENP.s = IPS_IDLE;
-        GuideNSN[0].value = GuideNSN[1].value = 0.0;
-        GuideWEN[0].value = GuideWEN[1].value = 0.0;
+        GuideNSNP.setState(IPS_IDLE);
+        GuideWENP.setState(IPS_IDLE);
+        GuideWENP[AXIS_RA].setValue(0);
+        GuideWENP[AXIS_DE].setValue(0);
+        GuideNSNP[AXIS_RA].setValue(0);
+        GuideNSNP[AXIS_DE].setValue(0);
 
         if (GuideNSTID)
         {
@@ -576,8 +587,8 @@ bool LX200Telescope::Abort()
         }
 
         LOG_INFO("Guide aborted.");
-        IDSetNumber(&GuideNSNP, nullptr);
-        IDSetNumber(&GuideWENP, nullptr);
+        GuideNSNP.apply();
+        GuideWENP.apply();
 
         return true;
     }
@@ -699,13 +710,16 @@ bool LX200Telescope::ISNewText(const char *dev, const char *name, char *texts[],
 
 bool LX200Telescope::ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
 {
+    // Check focuser interface
+    if (FI::processNumber(dev, name, values, names, n))
+        return true;
+    // Check guider interface
+    if (GI::processNumber(dev, name, values, names, n))
+        return true;
+
+    // Check rest of LX200Telescope properties
     if (dev != nullptr && strcmp(dev, getDeviceName()) == 0)
     {
-        if (strstr(name, "FOCUS_"))
-        {
-            return FI::processNumber(dev, name, values, names, n);
-        }
-
         // Update Frequency
         if (!strcmp(name, TrackFreqNP.name))
         {
@@ -750,36 +764,17 @@ bool LX200Telescope::ISNewNumber(const char *dev, const char *name, double value
             if (trackingMode != LX200_TRACK_MANUAL)
             {
                 trackingMode    = LX200_TRACK_MANUAL;
-                TrackModeS[0].s = ISS_OFF;
-                TrackModeS[1].s = ISS_OFF;
-                TrackModeS[2].s = ISS_OFF;
-                TrackModeS[3].s = ISS_ON;
-                TrackModeSP.s   = IPS_OK;
+                TrackModeSP[0].setState(ISS_OFF);
+                TrackModeSP[1].setState(ISS_OFF);
+                TrackModeSP[2].setState(ISS_OFF);
+                TrackModeSP[3].setState(ISS_ON);
+                TrackModeSP.setState(IPS_OK);
                 selectTrackingMode(PortFD, trackingMode);
-                IDSetSwitch(&TrackModeSP, nullptr);
+                TrackModeSP.apply();
             }
 
             return true;
         }
-
-        //        if (!strcmp(name, FocusTimerNP.name))
-        //        {
-        //            // Don't update if busy
-        //            if (FocusTimerNP.s == IPS_BUSY)
-        //                return true;
-
-        //            IUUpdateNumber(&FocusTimerNP, values, names, n);
-
-        //            FocusTimerNP.s = IPS_OK;
-
-        //            IDSetNumber(&FocusTimerNP, nullptr);
-
-        //            LOGF_DEBUG("Setting focus timer to %.2f", FocusTimerN[0].value);
-
-        //            return true;
-        //        }
-
-        processGuiderProperties(name, values, names, n);
     }
 
     //  if we didn't process it, continue up the chain, let somebody else
@@ -789,14 +784,12 @@ bool LX200Telescope::ISNewNumber(const char *dev, const char *name, double value
 
 bool LX200Telescope::ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
 {
+    // Focuser Interface
+    if (FI::processSwitch(dev, name, states, names, n))
+        return true;
+
     if (dev != nullptr && strcmp(dev, getDeviceName()) == 0)
     {
-        // Focuser
-        if (strstr(name, "FOCUS"))
-        {
-            return FI::processSwitch(dev, name, states, names, n);
-        }
-
         // Alignment
         if (!strcmp(name, AlignmentSP.name))
         {
@@ -853,79 +846,6 @@ bool LX200Telescope::ISNewSwitch(const char *dev, const char *name, ISState *sta
             return false;
         }
 
-        //        // Focus Motion
-        //        if (!strcmp(name, FocusMotionSP.name))
-        //        {
-        //            // If mode is "halt"
-        //            if (FocusModeS[0].s == ISS_ON)
-        //            {
-        //                FocusMotionSP.s = IPS_IDLE;
-        //                IDSetSwitch(&FocusMotionSP, "Focus mode is halt. Select slow or fast mode");
-        //                return true;
-        //            }
-
-        //            int last_motion = IUFindOnSwitchIndex(&FocusMotionSP);
-
-        //            if (IUUpdateSwitch(&FocusMotionSP, states, names, n) < 0)
-        //                return false;
-
-        //            index = IUFindOnSwitchIndex(&FocusMotionSP);
-
-        //            // If same direction and we're busy, stop
-        //            if (last_motion == index && FocusMotionSP.s == IPS_BUSY)
-        //            {
-        //                IUResetSwitch(&FocusMotionSP);
-        //                FocusMotionSP.s = IPS_IDLE;
-        //                setFocuserSpeedMode(PortFD, 0);
-        //                IDSetSwitch(&FocusMotionSP, nullptr);
-        //                return true;
-        //            }
-
-        //            if (!isSimulation() && setFocuserMotion(PortFD, index) < 0)
-        //            {
-        //                FocusMotionSP.s = IPS_ALERT;
-        //                IDSetSwitch(&FocusMotionSP, "Error setting focuser speed.");
-        //                return false;
-        //            }
-
-        //            // with a timer
-        //            if (FocusTimerN[0].value > 0)
-        //            {
-        //                FocusTimerNP.s  = IPS_BUSY;
-        //                FocusMotionSP.s = IPS_BUSY;
-        //                IEAddTimer(50, LX200Telescope::updateFocusHelper, this);
-        //            }
-
-        //            FocusMotionSP.s = IPS_OK;
-        //            IDSetSwitch(&FocusMotionSP, nullptr);
-        //            return true;
-        //        }
-
-        //        // Focus speed
-        //        if (!strcmp(name, FocusModeSP.name))
-        //        {
-        //            IUResetSwitch(&FocusModeSP);
-        //            IUUpdateSwitch(&FocusModeSP, states, names, n);
-
-        //            index = IUFindOnSwitchIndex(&FocusModeSP);
-
-        //            /* disable timer and motion */
-        //            if (index == 0)
-        //            {
-        //                IUResetSwitch(&FocusMotionSP);
-        //                FocusMotionSP.s = IPS_IDLE;
-        //                FocusTimerNP.s  = IPS_IDLE;
-        //                IDSetSwitch(&FocusMotionSP, nullptr);
-        //                IDSetNumber(&FocusTimerNP, nullptr);
-        //            }
-
-        //            if (!isSimulation())
-        //                setFocuserSpeedMode(PortFD, index);
-        //            FocusModeSP.s = IPS_OK;
-        //            IDSetSwitch(&FocusModeSP, nullptr);
-        //            return true;
-        //        }
-
         // Pulse-Guide command support
         if (!strcmp(name, UsePulseCmdSP.name))
         {
@@ -977,20 +897,21 @@ bool LX200Telescope::SetSlewRate(int index)
 
 bool LX200Telescope::updateSlewRate(int index)
 {
-    if (IUFindOnSwitchIndex(&SlewRateSP) == index)
+    if (SlewRateSP.findOnSwitchIndex() == index)
         return true;
 
     if (!isSimulation() && setSlewMode(PortFD, 3 - index) < 0)
     {
-        SlewRateSP.s = IPS_ALERT;
-        IDSetSwitch(&SlewRateSP, "Error setting slew mode.");
+        SlewRateSP.setState(IPS_ALERT);
+        LOG_ERROR("Error setting slew mode.");
+        SlewRateSP.apply();
         return false;
     }
 
-    IUResetSwitch(&SlewRateSP);
-    SlewRateS[index].s = ISS_ON;
-    SlewRateSP.s = IPS_OK;
-    IDSetSwitch(&SlewRateSP, nullptr);
+    SlewRateSP.reset();
+    SlewRateSP[index].setState(ISS_ON);
+    SlewRateSP.setState(IPS_OK);
+    SlewRateSP.apply();
     return true;
 }
 
@@ -1085,7 +1006,7 @@ void LX200Telescope::mountSim()
             break;
 
         case SCOPE_TRACKING:
-            switch (IUFindOnSwitchIndex(&TrackModeSP))
+            switch (TrackModeSP.findOnSwitchIndex())
             {
                 case TRACK_SIDEREAL:
                     da = 0;
@@ -1103,8 +1024,8 @@ void LX200Telescope::mountSim()
                     break;
 
                 case TRACK_CUSTOM:
-                    da = ((TrackRateN[AXIS_RA].value - TRACKRATE_SIDEREAL) / 3600.0 * dt / 15.);
-                    dx = (TrackRateN[AXIS_DE].value / 3600.0 * dt);
+                    da = ((TrackRateNP[AXIS_RA].getValue() - TRACKRATE_SIDEREAL) / 3600.0 * dt / 15.);
+                    dx = (TrackRateNP[AXIS_DE].getValue() / 3600.0 * dt);
                     break;
 
             }
@@ -1225,8 +1146,8 @@ void LX200Telescope::slewError(int slewCode)
     else
         LOGF_ERROR("Slew failed (%d).", slewCode);
 
-    EqNP.s = IPS_ALERT;
-    IDSetNumber(&EqNP, nullptr);
+    EqNP.setState(IPS_ALERT);
+    EqNP.apply();
 }
 
 void LX200Telescope::getAlignment()
@@ -1325,7 +1246,7 @@ bool LX200Telescope::sendScopeTime()
     {
         char utcStr[8] = {0};
         snprintf(utcStr, 8, "%.2f", offset);
-        IUSaveText(&TimeT[1], utcStr);
+        TimeTP[OFFSET].setText(utcStr);
     }
     else
     {
@@ -1376,14 +1297,14 @@ bool LX200Telescope::sendScopeTime()
 
     // Format it into the final UTC ISO 8601
     strftime(cdate, MAXINDINAME, "%Y-%m-%dT%H:%M:%S", &utm);
-    IUSaveText(&TimeT[0], cdate);
+    TimeTP[UTC].setText(cdate);
 
-    LOGF_DEBUG("Mount controller UTC Time: %s", TimeT[0].text);
-    LOGF_DEBUG("Mount controller UTC Offset: %s", TimeT[1].text);
+    LOGF_DEBUG("Mount controller UTC Time: %s", TimeTP[UTC].getText());
+    LOGF_DEBUG("Mount controller UTC Offset: %s", TimeTP[OFFSET].getText());
 
     // Let's send everything to the client
-    TimeTP.s = IPS_OK;
-    IDSetText(&TimeTP, nullptr);
+    TimeTP.setState(IPS_OK);
+    TimeTP.apply();
 
     return true;
 }
@@ -1397,11 +1318,11 @@ bool LX200Telescope::sendScopeLocation()
 
     if (isSimulation())
     {
-        LocationNP.np[LOCATION_LATITUDE].value = 29.5;
-        LocationNP.np[LOCATION_LONGITUDE].value = 48.0;
-        LocationNP.np[LOCATION_ELEVATION].value = 10;
-        LocationNP.s           = IPS_OK;
-        IDSetNumber(&LocationNP, nullptr);
+        LocationNP[LOCATION_LATITUDE].setValue(29.5);
+        LocationNP[LOCATION_LONGITUDE].setValue(48.0);
+        LocationNP[LOCATION_ELEVATION].setValue(10);
+        LocationNP.setState(IPS_OK);
+        LocationNP.apply( );
         return true;
     }
 
@@ -1412,8 +1333,10 @@ bool LX200Telescope::sendScopeLocation()
     }
     else
     {
+        double value = 0;
         snprintf(lat_sexagesimal, MAXINDIFORMAT, "%02d:%02d:%02.1lf", lat_dd, lat_mm, lat_ssf);
-        f_scansexa(lat_sexagesimal, &(LocationNP.np[LOCATION_LATITUDE].value));
+        f_scansexa(lat_sexagesimal, &value);
+        LocationNP[LOCATION_LATITUDE].setValue(value);
     }
 
     if (getSiteLongitude(PortFD, &long_dd, &long_mm, &long_ssf) < 0)
@@ -1423,17 +1346,21 @@ bool LX200Telescope::sendScopeLocation()
     }
     else
     {
+        double value = 0;
         snprintf(lng_sexagesimal, MAXINDIFORMAT, "%02d:%02d:%02.1lf", long_dd, long_mm, long_ssf);
-        f_scansexa(lng_sexagesimal, &(LocationNP.np[LOCATION_LONGITUDE].value));
+        f_scansexa(lng_sexagesimal, &value);
+        value = range360(value);
+        LocationNP[LOCATION_LONGITUDE].setValue(value);
+        fs_sexa(lng_sexagesimal, LocationNP[LOCATION_LONGITUDE].getValue(), 2, 36000);
     }
 
-    LOGF_INFO("Mount has Latitude %s (%g) Longitude %s (%g) (Longitude sign in carthography format)",
+    LOGF_INFO("Mount has Latitude %s (%g) Longitude (0 to +360 Eastwards) %s (%g)",
               lat_sexagesimal,
-              LocationN[LOCATION_LATITUDE].value,
+              LocationNP[LOCATION_LATITUDE].getValue(),
               lng_sexagesimal,
-              LocationN[LOCATION_LONGITUDE].value);
+              LocationNP[LOCATION_LONGITUDE].getValue());
 
-    IDSetNumber(&LocationNP, nullptr);
+    LocationNP.apply();
 
     saveConfig(true, "GEOGRAPHIC_COORD");
 
@@ -1449,7 +1376,7 @@ IPState LX200Telescope::GuideNorth(uint32_t ms)
     }
 
     // If we're using pulse command, then MovementXXX should NOT be active at all.
-    if (usePulseCommand && (MovementNSSP.s == IPS_BUSY || MovementWESP.s == IPS_BUSY))
+    if (usePulseCommand && (MovementNSSP.getState() == IPS_BUSY || MovementWESP.getState() == IPS_BUSY))
     {
         LOG_ERROR("Cannot pulse guide while manually in motion. Stop first.");
         return IPS_ALERT;
@@ -1470,8 +1397,8 @@ IPState LX200Telescope::GuideNorth(uint32_t ms)
         updateSlewRate(SLEW_GUIDE);
 
         ISState states[] = { ISS_ON, ISS_OFF };
-        const char *names[] = { MovementNSS[DIRECTION_NORTH].name, MovementNSS[DIRECTION_SOUTH].name};
-        ISNewSwitch(MovementNSSP.device, MovementNSSP.name, states, const_cast<char **>(names), 2);
+        const char *names[] = { MovementNSSP[DIRECTION_NORTH].getName(), MovementNSSP[DIRECTION_SOUTH].getName()};
+        ISNewSwitch(MovementNSSP.getDeviceName(), MovementNSSP.getName(), states, const_cast<char **>(names), 2);
     }
 
     guide_direction_ns = LX200_NORTH;
@@ -1488,7 +1415,7 @@ IPState LX200Telescope::GuideSouth(uint32_t ms)
     }
 
     // If we're using pulse command, then MovementXXX should NOT be active at all.
-    if (usePulseCommand && (MovementNSSP.s == IPS_BUSY || MovementWESP.s == IPS_BUSY))
+    if (usePulseCommand && (MovementNSSP.getState() == IPS_BUSY || MovementWESP.getState() == IPS_BUSY))
     {
         LOG_ERROR("Cannot pulse guide while manually in motion. Stop first.");
         return IPS_ALERT;
@@ -1509,8 +1436,8 @@ IPState LX200Telescope::GuideSouth(uint32_t ms)
         updateSlewRate(SLEW_GUIDE);
 
         ISState states[] = { ISS_OFF, ISS_ON };
-        const char *names[] = { MovementNSS[DIRECTION_NORTH].name, MovementNSS[DIRECTION_SOUTH].name};
-        ISNewSwitch(MovementNSSP.device, MovementNSSP.name, states, const_cast<char **>(names), 2);
+        const char *names[] = { MovementNSSP[DIRECTION_NORTH].getName(), MovementNSSP[DIRECTION_SOUTH].getName()};
+        ISNewSwitch(MovementNSSP.getDeviceName(), MovementNSSP.getName(), states, const_cast<char **>(names), 2);
     }
 
     guide_direction_ns = LX200_SOUTH;
@@ -1527,7 +1454,7 @@ IPState LX200Telescope::GuideEast(uint32_t ms)
     }
 
     // If we're using pulse command, then MovementXXX should NOT be active at all.
-    if (usePulseCommand && (MovementNSSP.s == IPS_BUSY || MovementWESP.s == IPS_BUSY))
+    if (usePulseCommand && (MovementNSSP.getState() == IPS_BUSY || MovementWESP.getState() == IPS_BUSY))
     {
         LOG_ERROR("Cannot pulse guide while manually in motion. Stop first.");
         return IPS_ALERT;
@@ -1548,8 +1475,8 @@ IPState LX200Telescope::GuideEast(uint32_t ms)
         updateSlewRate(SLEW_GUIDE);
 
         ISState states[] = { ISS_OFF, ISS_ON };
-        const char *names[] = { MovementWES[DIRECTION_WEST].name, MovementWES[DIRECTION_EAST].name};
-        ISNewSwitch(MovementWESP.device, MovementWESP.name, states, const_cast<char **>(names), 2);
+        const char *names[] = { MovementWESP[DIRECTION_WEST].getName(), MovementWESP[DIRECTION_EAST].getName()};
+        ISNewSwitch(MovementWESP.getDeviceName(), MovementWESP.getName(), states, const_cast<char **>(names), 2);
     }
 
     guide_direction_we = LX200_EAST;
@@ -1566,7 +1493,7 @@ IPState LX200Telescope::GuideWest(uint32_t ms)
     }
 
     // If we're using pulse command, then MovementXXX should NOT be active at all.
-    if (usePulseCommand && (MovementNSSP.s == IPS_BUSY || MovementWESP.s == IPS_BUSY))
+    if (usePulseCommand && (MovementNSSP.getState() == IPS_BUSY || MovementWESP.getState() == IPS_BUSY))
     {
         LOG_ERROR("Cannot pulse guide while manually in motion. Stop first.");
         return IPS_ALERT;
@@ -1587,8 +1514,8 @@ IPState LX200Telescope::GuideWest(uint32_t ms)
         updateSlewRate(SLEW_GUIDE);
 
         ISState states[] = { ISS_ON, ISS_OFF };
-        const char *names[] = { MovementWES[DIRECTION_WEST].name, MovementWES[DIRECTION_EAST].name};
-        ISNewSwitch(MovementWESP.device, MovementWESP.name, states, const_cast<char **>(names), 2);
+        const char *names[] = { MovementWESP[DIRECTION_WEST].getName(), MovementWESP[DIRECTION_EAST].getName()};
+        ISNewSwitch(MovementWESP.getDeviceName(), MovementWESP.getName(), states, const_cast<char **>(names), 2);
     }
 
     guide_direction_we = LX200_WEST;
@@ -1616,15 +1543,15 @@ void LX200Telescope::guideTimeoutWE()
     if (usePulseCommand == false)
     {
         ISState states[] = { ISS_OFF, ISS_OFF };
-        const char *names[] = { MovementWES[DIRECTION_WEST].name, MovementWES[DIRECTION_EAST].name};
-        ISNewSwitch(MovementWESP.device, MovementWESP.name, states, const_cast<char **>(names), 2);
+        const char *names[] = { MovementWESP[DIRECTION_WEST].getName(), MovementWESP[DIRECTION_EAST].getName()};
+        ISNewSwitch(MovementWESP.getDeviceName(), MovementWESP.getName(), states, const_cast<char **>(names), 2);
     }
 
-    GuideWENP.np[DIRECTION_WEST].value = 0;
-    GuideWENP.np[DIRECTION_EAST].value = 0;
-    GuideWENP.s           = IPS_IDLE;
-    GuideWETID            = 0;
-    IDSetNumber(&GuideWENP, nullptr);
+    GuideWENP[DIRECTION_WEST].setValue(0);
+    GuideWENP[DIRECTION_EAST].setValue(0);
+    GuideWENP.setState(IPS_IDLE);
+    GuideWETID = 0;
+    GuideWENP.apply();
 }
 
 void LX200Telescope::guideTimeoutNS()
@@ -1632,15 +1559,15 @@ void LX200Telescope::guideTimeoutNS()
     if (usePulseCommand == false)
     {
         ISState states[] = { ISS_OFF, ISS_OFF };
-        const char *names[] = { MovementNSS[DIRECTION_NORTH].name, MovementNSS[DIRECTION_SOUTH].name};
-        ISNewSwitch(MovementNSSP.device, MovementNSSP.name, states, const_cast<char **>(names), 2);
+        const char *names[] = { MovementNSSP[DIRECTION_NORTH].getName(), MovementNSSP[DIRECTION_SOUTH].getName()};
+        ISNewSwitch(MovementNSSP.getDeviceName(), MovementNSSP.getName(), states, const_cast<char **>(names), 2);
     }
 
-    GuideNSNP.np[0].value = 0;
-    GuideNSNP.np[1].value = 0;
-    GuideNSNP.s           = IPS_IDLE;
+    GuideNSNP[0].setValue(0);
+    GuideNSNP[1].setValue(0);
+    GuideNSNP.setState(IPS_IDLE);
     GuideNSTID            = 0;
-    IDSetNumber(&GuideNSNP, nullptr);
+    GuideNSNP.apply();
 
 }
 
