@@ -35,14 +35,15 @@ static std::unique_ptr<ScopeSim> telescope_sim(new ScopeSim());
 #define GUIDE_WEST  0
 #define GUIDE_EAST  1
 
-ScopeSim::ScopeSim()
+ScopeSim::ScopeSim(): GI(this)
 {
     DBG_SCOPE = static_cast<uint32_t>(INDI::Logger::getInstance().addDebugLevel("Scope Verbose", "SCOPE"));
 
     SetTelescopeCapability(TELESCOPE_CAN_PARK | TELESCOPE_CAN_SYNC | TELESCOPE_CAN_GOTO | TELESCOPE_CAN_ABORT |
                            TELESCOPE_HAS_TIME | TELESCOPE_HAS_LOCATION | TELESCOPE_HAS_TRACK_MODE | TELESCOPE_CAN_CONTROL_TRACK |
-                           TELESCOPE_HAS_TRACK_RATE,
-                           4);
+                           TELESCOPE_HAS_TRACK_RATE | TELESCOPE_CAN_HOME_FIND | TELESCOPE_CAN_HOME_SET | TELESCOPE_CAN_HOME_GO,
+                           4
+                          );
 
     /* initialize random seed: */
     srand(static_cast<uint32_t>(time(nullptr)));
@@ -65,48 +66,47 @@ bool ScopeSim::initProperties()
 
 #ifdef USE_SIM_TAB
     // mount type and alignment properties, these are in the Simulation tab
-    IUFillSwitch(&mountTypeS[Alignment::ALTAZ], "ALTAZ", "AltAz", ISS_OFF);
-    IUFillSwitch(&mountTypeS[Alignment::EQ_FORK], "EQ_FORK", "Fork (Eq)", ISS_OFF);
-    IUFillSwitch(&mountTypeS[Alignment::EQ_GEM], "EQ_GEM", "GEM", ISS_ON);
-    IUFillSwitchVector(&mountTypeSP, mountTypeS, 3, getDeviceName(), "MOUNT_TYPE", "Mount Type",
-                       "Simulation", IP_WO, ISR_1OFMANY, 60, IPS_IDLE );
+    mountTypeSP[ALTAZ].fill("ALTAZ", "ALTAZ", ISS_OFF);
+    mountTypeSP[EQ_FORK].fill("EQ_FORK", "Fork (Eq)", ISS_OFF);
+    mountTypeSP[EQ_GEM].fill("EQ_GEM", "GEM", ISS_OFF);
+    mountTypeSP.fill(getDeviceName(), "MOUNT_TYPE", "Mount Type", "Simulation", IP_WO, ISR_1OFMANY, 60, IPS_IDLE);
 
-    IUFillSwitch(&simPierSideS[0], "PS_OFF", "Off", ISS_OFF);
-    IUFillSwitch(&simPierSideS[1], "PS_ON", "On", ISS_ON);
-    IUFillSwitchVector(&simPierSideSP, simPierSideS, 2, getDeviceName(), "SIM_PIER_SIDE", "Sim Pier Side",
-                       "Simulation", IP_WO, ISR_1OFMANY, 60, IPS_IDLE );
+    simPierSideSP[PS_OFF].fill("PS_OFF", "Off", ISS_OFF);
+    simPierSideSP[PS_ON].fill("PS_ON", "On", ISS_ON);
+    simPierSideSP.fill(getDeviceName(), "SIM_PIER_SIDE", "Sim Pier Side",
+                       "Simulation", IP_WO, ISR_1OFMANY, 60, IPS_IDLE);
 
-    IUFillNumber(&mountModelN[0], "MM_IH", "Ha Zero (IH)", "%g", -5, 5, 0.01, 0);
-    IUFillNumber(&mountModelN[1], "MM_ID", "Dec Zero (ID)", "%g", -5, 5, 0.01, 0);
-    IUFillNumber(&mountModelN[2], "MM_CH", "Cone (CH)", "%g", -5, 5, 0.01, 0);
-    IUFillNumber(&mountModelN[3], "MM_NP", "Ha/Dec (NP)", "%g", -5, 5, 0.01, 0);
-    IUFillNumber(&mountModelN[4], "MM_MA", "Pole Azm (MA)", "%g", -5, 5, 0.01, 0);
-    IUFillNumber(&mountModelN[5], "MM_ME", "Pole elev (ME)", "%g", -5, 5, 0.01, 0);
-    IUFillNumberVector(&mountModelNP, mountModelN, 6, getDeviceName(), "MOUNT_MODEL", "Mount Model",
-                       "Simulation", IP_WO, 0, IPS_IDLE);
+    mountModelNP[MM_IH].fill("MM_IH", "Ha Zero (IH)", "%g", -5, 5, 0.01, 0);
+    mountModelNP[MM_ID].fill("MM_ID", "Dec Zero (ID)", "%g", -5, 5, 0.01, 0);
+    mountModelNP[MM_CH].fill("MM_CH", "Cone (CH)", "%g", -5, 5, 0.01, 0);
+    mountModelNP[MM_NP].fill("MM_NP", "Ha/Dec (NP)", "%g", -5, 5, 0.01, 0);
+    mountModelNP[MM_MA].fill("MM_MA", "Pole Azm (MA)", "%g", -5, 5, 0.01, 0);
+    mountModelNP[MM_ME].fill("MM_ME", "Pole elev (ME)", "%g", -5, 5, 0.01, 0);
+    mountModelNP.fill(getDeviceName(), "MOUNT_MODEL", "Mount Model",
+                      "Simulation", IP_WO, 0, IPS_IDLE);
 
-    IUFillNumber(&flipHourAngleN[0], "FLIP_HA", "Hour Angle (deg)", "%g", -20, 20, 0.1, 0);
-    IUFillNumberVector(&flipHourAngleNP, flipHourAngleN, 1, getDeviceName(), "FLIP_HA", "Flip Posn.",
-                       "Simulation", IP_WO, 0, IPS_IDLE);
+    flipHourAngleNP[0].fill("FLIP_HA", "Hour Angle (deg)", "%g", -20, 20, 0.1, 0);
+    flipHourAngleNP.fill(getDeviceName(), "FLIP_HA", "Flip Posn.",
+                         "Simulation", IP_WO, 0, IPS_IDLE);
 
-    IUFillNumber(&mountAxisN[0], "PRIMARY", "Primary (Ha)", "%g", -180, 180, 0.01, 0);
-    IUFillNumber(&mountAxisN[1], "SECONDARY", "Secondary (Dec)", "%g", -180, 180, 0.01, 0);
-    IUFillNumberVector(&mountAxisNP, mountAxisN, 2, getDeviceName(), "MOUNT_AXES", "Mount Axes",
-                       "Simulation", IP_RO, 0, IPS_IDLE);
+    mountAxisNP[PRIMARY].fill("PRIMARY", "Primary (Ha)", "%g", -180, 180, 0.01, 0);
+    mountAxisNP[SECONDARY].fill("SECONDARY", "Secondary (Dec)", "%g", -180, 180, 0.01, 0);
+    mountAxisNP.fill(getDeviceName(), "MOUNT_AXES", "Mount Axes",
+                     "Simulation", IP_RO, 0, IPS_IDLE);
 #endif
 
     /* How fast do we guide compared to sidereal rate */
-    IUFillNumber(&GuideRateN[RA_AXIS], "GUIDE_RATE_WE", "W/E Rate", "%g", 0, 1, 0.1, 0.5);
-    IUFillNumber(&GuideRateN[DEC_AXIS], "GUIDE_RATE_NS", "N/S Rate", "%g", 0, 1, 0.1, 0.5);
-    IUFillNumberVector(&GuideRateNP, GuideRateN, 2, getDeviceName(), "GUIDE_RATE", "Guiding Rate", MOTION_TAB, IP_RW, 0,
-                       IPS_IDLE);
+    GuideRateNP[RA_AXIS].fill("GUIDE_RATE_WE", "W/E Rate", "%g", 0, 1, 0.1, 0.5);
+    GuideRateNP[DEC_AXIS].fill("GUIDE_RATE_NS", "N/S Rate", "%g", 0, 1, 0.1, 0.5);
+    GuideRateNP.fill(getDeviceName(), "GUIDE_RATE", "Guiding Rate", MOTION_TAB, IP_RW, 0,
+                     IPS_IDLE);
 
-    IUFillSwitch(&SlewRateS[SLEW_GUIDE], "SLEW_GUIDE", "Guide", ISS_OFF);
-    IUFillSwitch(&SlewRateS[SLEW_CENTERING], "SLEW_CENTERING", "Centering", ISS_OFF);
-    IUFillSwitch(&SlewRateS[SLEW_FIND], "SLEW_FIND", "Find", ISS_OFF);
-    IUFillSwitch(&SlewRateS[SLEW_MAX], "SLEW_MAX", "Max", ISS_ON);
-    IUFillSwitchVector(&SlewRateSP, SlewRateS, 4, getDeviceName(), "TELESCOPE_SLEW_RATE", "Slew Rate", MOTION_TAB,
-                       IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
+    SlewRateSP[SLEW_GUIDE].fill("SLEW_GUIDE", "Guide", ISS_OFF);
+    SlewRateSP[SLEW_CENTERING].fill("SLEW_CENTERING", "Centering", ISS_OFF);
+    SlewRateSP[SLEW_FIND].fill("SLEW_FIND", "Find", ISS_OFF);
+    SlewRateSP[SLEW_MAX].fill("SLEW_MAX", "Max", ISS_ON);
+    SlewRateSP.fill(getDeviceName(), "TELESCOPE_SLEW_RATE", "Slew Rate", MOTION_TAB,
+                    IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
     // Add Tracking Modes, the order must match the order of the TelescopeTrackMode enum
     AddTrackMode("TRACK_SIDEREAL", "Sidereal", true);
@@ -117,7 +117,7 @@ bool ScopeSim::initProperties()
     // RA is a rotating frame, while HA or Alt/Az is not
     SetParkDataType(PARK_HA_DEC);
 
-    initGuiderProperties(getDeviceName(), MOTION_TAB);
+    GI::initProperties(MOTION_TAB);
 
     /* Add debug controls so we may debug driver if necessary */
     addDebugControl();
@@ -135,27 +135,16 @@ void ScopeSim::ISGetProperties(const char *dev)
     INDI::Telescope::ISGetProperties(dev);
 
 #ifdef USE_SIM_TAB
-    defineProperty(&mountTypeSP);
-    loadConfig(true, mountTypeSP.name);
-    defineProperty(&simPierSideSP);
-    loadConfig(true, simPierSideSP.name);
-    defineProperty(&mountModelNP);
-    loadConfig(true, mountModelNP.name);
-    defineProperty(&mountAxisNP);
-    defineProperty(&flipHourAngleNP);
-    loadConfig(true, flipHourAngleNP.name);
+    defineProperty(mountTypeSP);
+    mountTypeSP.load();
+    defineProperty(simPierSideSP);
+    simPierSideSP.load();
+    defineProperty(mountModelNP);
+    mountModelNP.load();
+    defineProperty(mountAxisNP);
+    defineProperty(flipHourAngleNP);
+    flipHourAngleNP.load();
 #endif
-    /*
-    if (isConnected())
-    {
-        defineProperty(&GuideNSNP);
-        defineProperty(&GuideWENP);
-        defineProperty(&GuideRateNP);
-        defineProperty(&EqPENV);
-        defineProperty(&PEErrNSSP);
-        defineProperty(&PEErrWESP);
-    }
-    */
 }
 
 bool ScopeSim::updateProperties()
@@ -166,10 +155,9 @@ bool ScopeSim::updateProperties()
 
     if (isConnected())
     {
-        defineProperty(&GuideNSNP);
-        defineProperty(&GuideWENP);
-        defineProperty(&GuideRateNP);
-        loadConfig(true, GuideRateNP.name);
+        defineProperty(GuideRateNP);
+        GuideRateNP.load();
+        //defineProperty(HomeSP);
 
         if (InitPark())
         {
@@ -178,10 +166,10 @@ bool ScopeSim::updateProperties()
             {
                 // at this point there is a valid ParkData.xml available
 
-                alignment.latitude = Angle(LocationN[LOCATION_LATITUDE].value);
-                alignment.longitude = Angle(LocationN[LOCATION_LONGITUDE].value);
-                currentRA = (alignment.lst() - Angle(ParkPositionN[AXIS_RA].value, Angle::ANGLE_UNITS::HOURS)).Hours();
-                currentDEC = ParkPositionN[AXIS_DE].value;
+                alignment.latitude = Angle(LocationNP[LOCATION_LATITUDE].getValue());
+                alignment.longitude = Angle(LocationNP[LOCATION_LONGITUDE].getValue());
+                currentRA = (alignment.lst() - Angle(ParkPositionNP[AXIS_RA].getValue(), Angle::ANGLE_UNITS::HOURS)).Hours();
+                currentDEC = ParkPositionNP[AXIS_DE].getValue();
                 Sync(currentRA, currentDEC);
 
             }
@@ -202,11 +190,12 @@ bool ScopeSim::updateProperties()
     }
     else
     {
-        deleteProperty(GuideNSNP.name);
-        deleteProperty(GuideWENP.name);
-        deleteProperty(GuideRateNP.name);
+        deleteProperty(GuideRateNP);
+        //deleteProperty(HomeSP);
     }
 
+    GI::updateProperties();
+
     return true;
 }
 
@@ -250,7 +239,7 @@ bool ScopeSim::ReadScopeStatus()
             if (!slewing)
             {
                 SetParked(true);
-                EqNP.s = IPS_IDLE;
+                EqNP.setState(IPS_IDLE);
                 LOG_INFO("Telescope slew is complete. Parked");
             }
             break;
@@ -261,8 +250,17 @@ bool ScopeSim::ReadScopeStatus()
                 // if the mount was not tracking before the slew should it remain not tracking?
                 TrackState = SCOPE_TRACKING;
                 SetTrackEnabled(true);
-                EqNP.s = IPS_IDLE;
-                LOG_INFO("Telescope slew is complete. Tracking...");
+                EqNP.setState(IPS_IDLE);
+
+                if (HomeSP.getState() == IPS_BUSY)
+                {
+                    HomeSP.reset();
+                    HomeSP.setState(IPS_OK);
+                    HomeSP.apply();
+                    LOG_INFO("Home position reached.");
+                }
+                else
+                    LOG_INFO("Telescope slew is complete. Tracking...");
 
                 // check the slew accuracy
                 auto dRa = targetRA - currentRA;
@@ -276,16 +274,16 @@ bool ScopeSim::ReadScopeStatus()
 
     if (guidingEW && !axisPrimary.IsGuiding())
     {
-        GuideWENP.np[0].value = 0;
-        GuideWENP.np[1].value = 0;
+        GuideWENP[0].setValue(0);
+        GuideWENP[1].setValue(0);
         GuideComplete(INDI_EQ_AXIS::AXIS_RA);
         guidingEW = false;
     }
 
     if (guidingNS && !axisSecondary.IsGuiding())
     {
-        GuideNSNP.np[0].value = 0;
-        GuideNSNP.np[1].value = 0;
+        GuideNSNP[0].setValue(0);
+        GuideNSNP[1].setValue(0);
         GuideComplete(INDI_EQ_AXIS::AXIS_DE);
         guidingNS = false;
     }
@@ -294,16 +292,16 @@ bool ScopeSim::ReadScopeStatus()
     double axisRA = axisPrimary.position.Degrees();
     double axisDE = axisSecondary.position.Degrees();
     // No need to spam log until we have some actual changes.
-    if (std::fabs(mountAxisN[AXIS_RA].value - axisRA) > 0.0001 ||
-            std::fabs(mountAxisN[AXIS_DE].value - axisDE) > 0.0001)
+    if (std::fabs(mountAxisNP[AXIS_RA].getValue() - axisRA) > 0.0001 ||
+            std::fabs(mountAxisNP[AXIS_DE].getValue() - axisDE) > 0.0001)
     {
-        mountAxisN[AXIS_RA].value = axisRA;
-        mountAxisN[AXIS_DE].value = axisDE;
+        mountAxisNP[AXIS_RA].setValue(axisRA);
+        mountAxisNP[AXIS_DE].setValue(axisDE);
 
         LOGF_EXTRA1("%s: %f, ra %f", axisPrimary.axisName, axisPrimary.position.Degrees(), ra.Hours());
         LOGF_EXTRA1("%s: %f, dec %f", axisSecondary.axisName, axisSecondary.position.Degrees(), dec.Degrees());
 
-        IDSetNumber(&mountAxisNP, nullptr);
+        mountAxisNP.apply();
     }
 #endif
 
@@ -339,7 +337,7 @@ bool ScopeSim::Sync(double ra, double dec)
 
     LOG_INFO("Sync is successful.");
 
-    EqNP.s = IPS_OK;
+    EqNP.setState(IPS_OK);
 
     NewRaDec(currentRA, currentDEC);
 
@@ -394,44 +392,39 @@ bool ScopeSim::UnPark()
 
 bool ScopeSim::ISNewNumber(const char *dev, const char *name, double values[], char *names[], int n)
 {
-    //  first check if it's for our device
+    // Check guider interface
+    if (GI::processNumber(dev, name, values, names, n))
+        return true;
 
     if (dev != nullptr && strcmp(dev, getDeviceName()) == 0)
     {
         if (strcmp(name, "GUIDE_RATE") == 0)
         {
-            IUUpdateNumber(&GuideRateNP, values, names, n);
-            GuideRateNP.s = IPS_OK;
-            IDSetNumber(&GuideRateNP, nullptr);
-            return true;
-        }
-
-        if (strcmp(name, GuideNSNP.name) == 0 || strcmp(name, GuideWENP.name) == 0)
-        {
-            processGuiderProperties(name, values, names, n);
+            GuideRateNP.update(values, names, n);
+            GuideRateNP.setState(IPS_OK);
+            GuideRateNP.apply();
             return true;
         }
 
 #ifdef USE_SIM_TAB
-        if (strcmp(name, mountModelNP.name) == 0)
+        if (mountModelNP.isNameMatch(name))
         {
-            IUUpdateNumber(&mountModelNP, values, names, n);
-            mountModelNP.s = IPS_OK;
-            IDSetNumber(&mountModelNP, nullptr);
-            alignment.setCorrections(mountModelN[0].value, mountModelN[1].value,
-                                     mountModelN[2].value, mountModelN[3].value,
-                                     mountModelN[4].value, mountModelN[5].value);
-
-            saveConfig(true, mountModelNP.name);
+            mountModelNP.update(values, names, n);
+            mountModelNP.setState(IPS_OK);
+            mountModelNP.apply();
+            alignment.setCorrections(mountModelNP[MM_IH].getValue(), mountModelNP[MM_ID].getValue(),
+                                     mountModelNP[MM_CH].getValue(), mountModelNP[MM_NP].getValue(),
+                                     mountModelNP[MM_MA].getValue(), mountModelNP[MM_ME].getValue());
+            saveConfig(true, mountModelNP.getName());
             return true;
         }
 
-        if (strcmp(name, flipHourAngleNP.name) == 0)
+        if (flipHourAngleNP.isNameMatch(name))
         {
-            IUUpdateNumber(&flipHourAngleNP, values, names, n);
-            flipHourAngleNP.s = IPS_OK;
-            IDSetNumber(&flipHourAngleNP, nullptr);
-            alignment.setFlipHourAngle(flipHourAngleN[0].value);
+            flipHourAngleNP.update(values, names, n);
+            flipHourAngleNP.setState(IPS_OK);
+            flipHourAngleNP.apply();
+            alignment.setFlipHourAngle(flipHourAngleNP[0].getValue());
             return true;
         }
 #endif
@@ -447,35 +440,35 @@ bool ScopeSim::ISNewSwitch(const char *dev, const char *name, ISState *states, c
     if (dev != nullptr && strcmp(dev, getDeviceName()) == 0)
     {
 #ifdef USE_SIM_TAB
-        if (strcmp(name, mountTypeSP.name) == 0)
+        if (mountTypeSP.isNameMatch(name))
         {
-            if (IUUpdateSwitch(&mountTypeSP, states, names, n) < 0)
+            if (!mountTypeSP.update(states, names, n))
                 return false;
 
-            mountTypeSP.s = IPS_OK;
-            IDSetSwitch(&mountTypeSP, nullptr);
+            mountTypeSP.setState(IPS_OK);
+            mountTypeSP.apply();
             updateMountAndPierSide();
             return true;
         }
-        if (strcmp(name, simPierSideSP.name) == 0)
+        if (simPierSideSP.isNameMatch(name))
         {
-            if (IUUpdateSwitch(&simPierSideSP, states, names, n) < 0)
+            if (!simPierSideSP.update(states, names, n))
                 return false;
 
-            simPierSideSP.s = IPS_OK;
-            IDSetSwitch(&simPierSideSP, nullptr);
+            simPierSideSP.setState(IPS_OK);
+            simPierSideSP.apply();
             updateMountAndPierSide();
             return true;
         }
-#endif
-        // Slew mode
-        if (strcmp(name, SlewRateSP.name) == 0)
+#endif \
+    // Slew mode
+        if (SlewRateSP.isNameMatch(name))
         {
-            if (IUUpdateSwitch(&SlewRateSP, states, names, n) < 0)
+            if (SlewRateSP.update( states, names, n) == false)
                 return false;
 
-            SlewRateSP.s = IPS_OK;
-            IDSetSwitch(&SlewRateSP, nullptr);
+            SlewRateSP.setState(IPS_OK);
+            SlewRateSP.apply();
             return true;
         }
     }
@@ -498,7 +491,8 @@ bool ScopeSim::MoveNS(INDI_DIR_NS dir, TelescopeMotionCommand command)
         LOG_ERROR("Please unpark the mount before issuing any motion commands.");
         return false;
     }
-    mcRate = static_cast<int>(IUFindOnSwitchIndex(&SlewRateSP)) + 1;
+    mcRate = static_cast<int>(SlewRateSP.findOnSwitchIndex()) + 1;
+    mcRate = std::max(1,std::min(4,mcRate));
 
     int rate = (dir == INDI_DIR_NS::DIRECTION_NORTH) ? mcRate : -mcRate;
     LOGF_DEBUG("MoveNS dir %s, motion %s, rate %d", dir == DIRECTION_NORTH ? "N" : "S", command == 0 ? "start" : "stop", rate);
@@ -516,7 +510,9 @@ bool ScopeSim::MoveWE(INDI_DIR_WE dir, TelescopeMotionCommand command)
         return false;
     }
 
-    mcRate = static_cast<int>(IUFindOnSwitchIndex(&SlewRateSP)) + 1;
+    mcRate = static_cast<int>(SlewRateSP.findOnSwitchIndex()) + 1;
+    mcRate = std::max(1,std::min(4,mcRate));
+
     int rate = (dir == INDI_DIR_WE::DIRECTION_EAST) ? -mcRate : mcRate;
     LOGF_DEBUG("MoveWE dir %d, motion %s, rate %d", dir == DIRECTION_EAST ? "E" : "W", command == 0 ? "start" : "stop", rate);
 
@@ -526,7 +522,7 @@ bool ScopeSim::MoveWE(INDI_DIR_WE dir, TelescopeMotionCommand command)
 
 IPState ScopeSim::GuideNorth(uint32_t ms)
 {
-    double rate = GuideRateN[DEC_AXIS].value;
+    double rate = GuideRateNP[DEC_AXIS].getValue();
     axisSecondary.StartGuide(rate, ms);
     guidingNS = true;
     return IPS_BUSY;
@@ -534,7 +530,7 @@ IPState ScopeSim::GuideNorth(uint32_t ms)
 
 IPState ScopeSim::GuideSouth(uint32_t ms)
 {
-    double rate = GuideRateN[DEC_AXIS].value;
+    double rate = GuideRateNP[DEC_AXIS].getValue();
     axisSecondary.StartGuide(-rate, ms);
     guidingNS = true;
     return IPS_BUSY;
@@ -542,7 +538,7 @@ IPState ScopeSim::GuideSouth(uint32_t ms)
 
 IPState ScopeSim::GuideEast(uint32_t ms)
 {
-    double rate = GuideRateN[RA_AXIS].value;
+    double rate = GuideRateNP[RA_AXIS].getValue();
     axisPrimary.StartGuide(-rate, ms);
     guidingEW = true;
     return IPS_BUSY;
@@ -550,7 +546,7 @@ IPState ScopeSim::GuideEast(uint32_t ms)
 
 IPState ScopeSim::GuideWest(uint32_t ms)
 {
-    double rate = GuideRateN[RA_AXIS].value;
+    double rate = GuideRateNP[RA_AXIS].getValue();
     axisPrimary.StartGuide(rate, ms);
     guidingEW = true;
     return IPS_BUSY;
@@ -593,7 +589,7 @@ bool ScopeSim::SetTrackMode(uint8_t mode)
             axisSecondary.TrackRate(Axis::OFF);
             return true;
         case TRACK_CUSTOM:
-            SetTrackRate(TrackRateN[AXIS_RA].value, TrackRateN[AXIS_DE].value);
+            SetTrackRate(TrackRateNP[AXIS_RA].getValue(), TrackRateNP[AXIS_DE].getValue());
             return true;
     }
     return false;
@@ -618,11 +614,12 @@ bool ScopeSim::saveConfigItems(FILE *fp)
     INDI::Telescope::saveConfigItems(fp);
 
 #ifdef USE_SIM_TAB
-    IUSaveConfigNumber(fp, &GuideRateNP);
-    IUSaveConfigSwitch(fp, &mountTypeSP);
-    IUSaveConfigSwitch(fp, &simPierSideSP);
-    IUSaveConfigNumber(fp, &mountModelNP);
-    IUSaveConfigNumber(fp, &flipHourAngleNP);
+    GuideRateNP.save(fp);
+    mountTypeSP.save(fp);
+    simPierSideSP.save(fp);
+    mountModelNP.save(fp);
+    flipHourAngleNP.save(fp);
+
 #endif
     return true;
 }
@@ -641,8 +638,8 @@ bool ScopeSim::updateLocation(double latitude, double longitude, double elevatio
 bool ScopeSim::updateMountAndPierSide()
 {
 #ifdef USE_SIM_TAB
-    int mountType = IUFindOnSwitchIndex(&mountTypeSP);
-    int pierSide = IUFindOnSwitchIndex(&simPierSideSP);
+    int mountType = mountTypeSP.findOnSwitchIndex();
+    int pierSide = simPierSideSP.findOnSwitchIndex();
     if (mountType < 0 || pierSide < 0)
         return false;
 
@@ -680,5 +677,43 @@ bool ScopeSim::updateMountAndPierSide()
     return true;
 }
 
-/////////////////////////////////////////////////////////////////////
 
+IPState ScopeSim::ExecuteHomeAction(TelescopeHomeAction action)
+{
+    switch (action)
+    {
+        case HOME_FIND:
+            LOG_INFO("Finding home position...");
+            m_Home[AXIS_RA] = alignment.lst().Hours();
+            m_Home[AXIS_DE] = LocationNP[LOCATION_LATITUDE].getValue() > 0 ? 90 : -90;
+            Goto(m_Home[AXIS_RA], m_Home[AXIS_DE]);
+            return IPS_BUSY;
+
+        case HOME_SET:
+            m_Home[AXIS_RA] = (alignment.lst() - Angle(EqNP[AXIS_RA].getValue(), Angle::HOURS)).HoursHa();
+            m_Home[AXIS_DE] = EqNP[AXIS_DE].getValue();
+            LOG_INFO("Setting home position to current position.");
+            return IPS_OK;
+
+        case HOME_GO:
+        {
+            LOG_INFO("Going to home position.");
+            if (m_Home[AXIS_RA] == 0)
+            {
+                m_Home[AXIS_RA] = 0;
+                m_Home[AXIS_DE] = LocationNP[LOCATION_LATITUDE].getValue() > 0 ? 90 : -90;
+            }
+
+            // Home HA to home RA
+            auto ra = (alignment.lst() - Angle(m_Home[AXIS_RA], Angle::HOURS)).Hours();
+            Goto(ra, m_Home[AXIS_DE]);
+            return IPS_BUSY;
+        }
+
+        default:
+            return IPS_ALERT;
+    }
+    return IPS_ALERT;
+}
+
+/////////////////////////////////////////////////////////////////////
