@@ -50,7 +50,7 @@
 #include <string.h>
 #include <time.h>
 
-#ifdef __linux__
+#if defined(__linux__) || defined(__NetBSD__)
 #include <sys/ioctl.h>
 #endif
 
@@ -793,7 +793,7 @@ int tty_connect(const char *device, int bit_rate, int word_size, int parity, int
     case 57600:  bps = B57600;  break;
     case 115200: bps = B115200; break;
     case 230400: bps = B230400; break;
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
+#if !defined(__APPLE__) && !defined(__FreeBSD__) && !defined(__NetBSD__)
     case 460800: bps = B460800; break;
     case 576000: bps = B576000; break;
     case 921600: bps = B921600; break;
@@ -1252,6 +1252,16 @@ double range360(double r)
     return res;
 }
 
+double range180(double r)
+{
+    double res = r;
+    while (res < -180.0)
+        res += 360.0;
+    while (res > 180.0)
+        res -= 360.0;
+    return res;
+}
+
 double rangeDec(double decdegrees)
 {
     if ((decdegrees >= 270.0) && (decdegrees <= 360.0))
